@@ -1,5 +1,5 @@
 #
-#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 Olivier Sessink
+#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011, 2018 Olivier Sessink
 #All rights reserved.
 #
 #Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,8 @@
 #POSSIBILITY OF SUCH DAMAGE.
 #
 
+from __future__ import print_function
+
 import os.path
 import string
 #import os
@@ -38,6 +40,9 @@ import shutil
 import glob
 import subprocess
 
+
+dir_mode = 493								#octal 755, "rwxr-xr-x"
+
 statcache = {}
 
 def cachedlstat(path):
@@ -50,7 +55,7 @@ def cachedlstat(path):
 #	#if (path[-1:] == '/'):
 #	#	path = path[:-1]
 #	try:
-#		#print 'path='+path
+#		#print('path='+path)
 #		indx = string.rindex(path,'/')
 #		if (indx > 0):
 #			return path[:indx]
@@ -81,7 +86,7 @@ def path_is_safe(path, failquiet=0):
 		if (stat.S_ISLNK(statbuf[stat.ST_MODE])):
 			# Fedora has moved /sbin /lib and /bin into /usr
 			target = os.readlink(path)
-			print target, path
+			print(str(target) + str(path))
 			if (target[:4] != 'usr/'):
 				sys.stderr.write('ERROR: '+path+' is a symlink, please point to the real directory\n')
 				return -2
@@ -102,10 +107,10 @@ def chroot_is_safe(path, failquiet=0):
 			return retval
 	npath = os.path.dirname(path)
 	while (npath != '/'):
-		#print npath
+		#print(npath)
 		retval = path_is_safe(npath,0)
 		if (retval != 1):
-#			print 'testing path='+npath+'returned '+str(retval)
+#			print('testing path='+npath+'returned '+str(retval))
 			return retval
 		npath = os.path.dirname(npath)
 	return 1
@@ -130,9 +135,11 @@ def lddlist_libraries_linux(executable):
 	line = p.stdout.readline()
 	#pd = os.popen3('ldd '+executable)
 	#line = pd[1].readline()
+	if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+		line = line.decode('utf-8', 'replace')
 	while (len(line)>0):
-		subl = string.split(line)
-		#print 'parse line',subl,'with len',len(subl)
+		subl = line.split()
+		#print('parse line',subl,'with len',len(subl))
 		if (len(subl)>0):
 			if (subl[0] == 'statically' and subl[1] == 'linked'):
 				return retval
@@ -146,19 +153,21 @@ def lddlist_libraries_linux(executable):
 				if (os.path.exists(subl[2])):
 					retval += [subl[2]]
 				else:
-					print 'ldd returns non existing library '+subl[2]+' for '+executable
+					print('ldd returns non existing library '+subl[2]+' for '+executable)
 			# on gentoo amd64 the last entry of ldd looks like '/lib64/ld-linux-x86-64.so.2 (0x0000002a95556000)'
 			elif (len(subl)>=1 and subl[0][0] == '/'):
 				if (os.path.exists(subl[0])):
 					retval += [subl[0]]
 				else:
-					print 'ldd returns non existing library '+subl[0]
+					print('ldd returns non existing library '+subl[0])
 			else:
-				print 'WARNING: failed to parse ldd output '+line[:-1]
+				print('WARNING: failed to parse ldd output '+line[:-1])
 		else:
-			print 'WARNING: failed to parse ldd output '+line[:-1]
+			print('WARNING: failed to parse ldd output '+line[:-1])
 		#line = pd[1].readline()
 		line = p.stdout.readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 	return retval
 
 def lddlist_libraries_openbsd(executable):
@@ -169,8 +178,10 @@ def lddlist_libraries_openbsd(executable):
 	line = p.stdout.readline()
 #	pd = os.popen3('ldd '+executable)
 #	line = pd[1].readline()
+	if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+		line = line.decode('utf-8', 'replace')
 	while (len(line)>0):
-		subl = string.split(line)
+		subl = line.split()
 		if (len(subl)>0):
 			if (subl[0] == executable+':'):
 				pass
@@ -183,20 +194,22 @@ def lddlist_libraries_openbsd(executable):
 					if (os.path.exists(subl[4])):
 						retval += [subl[4]]
 					else:
-						print 'ldd returns non existing library '+subl[4]
+						print('ldd returns non existing library '+subl[4])
 				elif (mode == 4):
 					if (os.path.exists(subl[6])):
 						retval += [subl[6]]
 					else:
-						print 'ldd returns non existing library '+subl[6]
+						print('ldd returns non existing library '+subl[6])
 				else:
-					print 'unknown mode, please report this bug in jk_lib.py'
+					print('unknown mode, please report this bug in jk_lib.py')
 			else:
-				print 'WARNING: failed to parse ldd output '+line[:-1]
+				print('WARNING: failed to parse ldd output '+line[:-1])
 		else:
-			print 'WARNING: failed to parse ldd output '+line[:-1]
+			print('WARNING: failed to parse ldd output '+line[:-1])
 		line = p.stdout.readline()
 		#line = pd[1].readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 	return retval
 
 def lddlist_libraries_freebsd(executable):
@@ -206,8 +219,10 @@ def lddlist_libraries_freebsd(executable):
 	line = p.stdout.readline()
 	#pd = os.popen3('ldd '+executable)
 	#line = pd[1].readline()
+	if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+		line = line.decode('utf-8', 'replace')
 	while (len(line)>0):
-		subl = string.split(line)
+		subl = line.split()
 		if (len(subl)>0):
 			if (len(subl)==1 and subl[0][:len(executable)+1] == executable+':'):
 				pass
@@ -217,15 +232,17 @@ def lddlist_libraries_freebsd(executable):
 				if (os.path.exists(subl[2])):
 					retval += [subl[2]]
 				else:
-					print 'ldd returns non existing library '+subl[2]
+					print('ldd returns non existing library '+subl[2])
 			else:
-				print 'WARNING: failed to parse ldd output "'+line[:-1]+'"'
+				print('WARNING: failed to parse ldd output "'+line[:-1]+'"')
 		elif (line[:len(executable)+1] == executable+':'):
 			pass
 		else:
-			print 'WARNING: failed to parse ldd output "'+line[:-1]+'"'
+			print('WARNING: failed to parse ldd output "'+line[:-1]+'"')
 		line = p.stdout.readline()
 		#line = pd[1].readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 	return retval
 
 def lddlist_libraries(executable):
@@ -254,10 +271,10 @@ def resolve_realpath(path, chroot='', include_file=0):
 		spath = spath[:-1] 
 	ret = '/'
 	doscounter=0# a symlink loop may otherwise hang this script
-	#print 'path',path,'spath',spath
+	#print('path' + path + 'spath' + spath)
 	for entry in spath:
 		ret = os.path.join(ret,entry)
-		#print 'lstat',ret
+		#print('lstat' + ret)
 		sb = cachedlstat(ret)
 		if (stat.S_ISLNK(sb.st_mode)):
 			doscounter+=1
@@ -268,7 +285,7 @@ def resolve_realpath(path, chroot='', include_file=0):
 				tmp = os.path.normpath(os.path.join(os.path.dirname(ret),realpath))
 				if (len(chroot)>0 and tmp[:len(chroot)]!=chroot):
 					sys.stderr.write('ERROR: symlink '+tmp+' points outside jail, ABORT\n')
-					raise Exception, "Symlink points outside jail"
+					raise Exception("Symlink points outside jail")
 				ret = tmp
 	return os.path.join(ret,basename)
 
@@ -285,7 +302,7 @@ def OLDresolve_realpath(path, chroot='', include_file=0):
 		todopath = os.path.dirname(path)
 	doscounter=0 # a symlink loop may otherwise hang this script
 	while (todopath != '/' and doscounter != 100):
-		#print 'todopath=',todopath,'donepath=',donepath
+		#print('todopath=' + todopath + 'donepath=' + donepath)
 		sb = cachedlstat(todopath)
 		if (stat.S_ISLNK(sb.st_mode)):
 			doscounter += 1
@@ -298,7 +315,7 @@ def OLDresolve_realpath(path, chroot='', include_file=0):
 				tmp = os.path.normpath(os.path.join(os.path.dirname(todopath),realpath))
 				if (chrootlen>0 and tmp[:chrootlen]!=chroot):
 					sys.stderr.write('ERROR: symlink '+tmp+' points outside jail, ABORT\n')
-					raise Exception, "Symlink points outside jail" 
+					raise Exception("Symlink points outside jail")
 				todopath=tmp
 		else:
 			donepath = os.path.basename(todopath)+'/'+donepath
@@ -315,11 +332,11 @@ def copy_time_and_permissions(src, dst, be_verbose=0, allow_suid=0, copy_ownersh
 	mode = stat.S_IMODE(sbuf[stat.ST_MODE])
 	if (not allow_suid):
 		if (mode & (stat.S_ISUID | stat.S_ISGID)):
-			print 'removing setuid and setgid permissions from '+dst
-#			print 'setuid!!! mode='+str(mode)
+			print('removing setuid and setgid permissions from '+dst)
+#			print('setuid!!! mode='+str(mode))
 			mode = (mode & ~stat.S_ISUID) & ~stat.S_ISGID
-#			print 'setuid!!! after mode='+str(mode)
-#		print 'mode='+str(mode)
+#			print('setuid!!! after mode='+str(mode))
+#		print('mode='+str(mode))
 	os.utime(dst, (sbuf[stat.ST_ATIME], sbuf[stat.ST_MTIME]))
 	if (copy_ownership):
 		os.chown(dst, sbuf[stat.ST_UID], sbuf[stat.ST_GID])
@@ -344,12 +361,12 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 		chrootdirectory = resolve_realpath(chroot+directory,chroot)
 		if (os.path.exists(chrootdirectory)):
 			return
-	except OSError (errno,strerror):
+	except OSError:
 		pass
 	tmp = return_existing_base_directory(chroot+directory)
 	oldindx = len(tmp)-len(chroot)
 	# find the first slash after the existing directories
-	indx = string.find(directory,'/',oldindx+1)
+	indx = directory.find('/',oldindx+1)
 	if (indx == -1 ):
 		indx=len(directory)
 	while (indx != -1):
@@ -359,7 +376,8 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 		else:
 			try:
 				sb = cachedlstat(directory[:indx])
-			except OSError, (errno,strerror):
+			except OSError as e:
+				_, strerror = e.args
 				sys.stderr.write('ERROR: failed to lstat('+directory[:indx]+'):'+strerror+'\n')
 				break
 			if (stat.S_ISLNK(sb.st_mode)):
@@ -367,10 +385,11 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 				realfile = os.readlink(directory[:indx])
 				chrootname = resolve_realpath(chroot+directory[:indx],chroot)
 				if (be_verbose):
-					print 'Creating symlink '+chrootname+' to '+realfile
+					print('Creating symlink '+chrootname+' to '+realfile)
 				try:
 					os.symlink(realfile, chrootname)
-				except OSError, (errno,strerror):
+				except OSError as e:
+					errno, _ = e.args
 					if (errno == 17): # file exists
 						pass
 					else:
@@ -379,23 +398,24 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 					create_parent_path(chroot, realfile, be_verbose, copy_permissions, allow_suid, copy_ownership)
 				else:
 					indx2 = string.rfind(directory[:indx],'/')
-#					print 'try',directory[:indx2+1]+realfile
+#					print('try' + directory[:indx2+1]+realfile)
 					create_parent_path(chroot, directory[:indx2+1]+realfile, be_verbose, copy_permissions, allow_suid, copy_ownership)
 			elif (stat.S_ISDIR(sb.st_mode)):
 				chrootname = resolve_realpath(chroot+directory[:indx],chroot)
 				if (be_verbose):
-					print 'Creating directory '+chrootname
-				os.mkdir(chrootname, 0755)
+					print('Creating directory '+chrootname)
+				os.mkdir(chrootname, dir_mode)
 				if (copy_permissions):
 					try:
 						copy_time_and_permissions(directory[:indx], chrootname, be_verbose, allow_suid, copy_ownership)
-					except OSError, (errno,strerror):
+					except OSError as e:
+						_, strerror = e.args
 						sys.stderr.write('ERROR: failed to copy time/permissions/owner from '+directory[:indx]+' to '+chrootname+': '+strerror+'\n')
 			oldindx = indx
 		if (indx==len(directory)):
 			indx=-1
 		else:
-			indx = string.find(directory,'/',oldindx+1)
+			indx = directory.find('/',oldindx+1)
 			if (indx==-1):
 				indx=len(directory)
 
@@ -428,7 +448,7 @@ def join_path(spath):
 	return ret
 
 def create_parent_path(chroot,path,be_verbose=0, copy_permissions=1, allow_suid=0, copy_ownership=0):
-	#print 'create_parent_path, path',path,'in chroot',chroot
+	#print('create_parent_path, path' + path + 'in chroot' + chroot)
 	# the first part of the function checks the already existing paths in the jail
 	# and follows any symlinks relative to the jail
 	spath = split_path(path)
@@ -437,53 +457,55 @@ def create_parent_path(chroot,path,be_verbose=0, copy_permissions=1, allow_suid=
 	while (i<len(spath)):
 		tmp1 = os.path.join(existpath,spath[i])
 		if not os.path.exists(tmp1):
-			#print 'tmp1 does not exist',tmp1
+			#print('tmp1 does not exist' + tmp1)
 			break
 		tmp = resolve_realpath(tmp1,chroot,1)
 		if not os.path.exists(tmp):
-			#print 'tmp does not exist',tmp
+			#print('tmp does not exist' + tmp)
 			break
-		#print 'tmp exists:',tmp
+		#print('tmp exists:' + tmp)
 		existpath = tmp
 		i+=1
-	#print 'existpath result:',existpath,'i=',i
+	#print('existpath result:' + existpath + 'i=' + i)
 	# the second part of the function creates the missing parts in the jail
 	# according to the original directory names, including any symlinks
 	while (i<len(spath)):
 		origpath = join_path(spath[0:i+1])
 		jailpath = os.path.join(existpath,spath[i])
-		#print 'origpath',origpath,'jailpath',jailpath
+		#print('origpath' + origpath + 'jailpath' + jailpath)
 		try:
 			sb = cachedlstat(origpath)
-		except OSError, (errno,strerror):
+		except OSError as e:
+			_, strerror = e.args
 			sys.stderr.write('ERROR: failed to lstat('+origpath+'):'+strerror+'\n')
 			return None
 		if (stat.S_ISDIR(sb.st_mode)):
 			if (be_verbose):
-				print 'Create directory '+jailpath
-			os.mkdir(jailpath, 0755)
+				print('Create directory '+jailpath)
+			os.mkdir(jailpath, dir_mode)
 			if (copy_permissions):
 				try:
 					copy_time_and_permissions(origpath, jailpath, be_verbose, allow_suid, copy_ownership)
-				except OSError, (errno,strerror):
+				except OSError as e:
+					_, strerror = e.args
 					sys.stderr.write('ERROR: failed to copy time/permissions/owner from '+directory[:indx]+' to '+chrootname+': '+strerror+'\n')
 		elif (stat.S_ISLNK(sb.st_mode)):
 			realfile = os.readlink(origpath)
 			if (be_verbose):
-				print 'Creating symlink '+jailpath+' to '+realfile
+				print('Creating symlink '+jailpath+' to '+realfile)
 			os.symlink(realfile,jailpath)
 			if (realfile[0]=='/'):
 				jailpath = create_parent_path(chroot, realfile,be_verbose, copy_permissions, allow_suid, copy_ownership)
 			else:
 				tmp = os.path.normpath(os.path.join(os.path.dirname(jailpath),realfile))
-				#print 'symlink resolves to ',tmp
+				#print('symlink resolves to ' + tmp)
 				if (len(chroot)>0 and tmp[:len(chroot)]!=chroot):
 					sys.stderr.write('ERROR: symlink '+tmp+' points outside jail, ABORT\n')
-					raise Exception, "Symlink points outside jail"
+					raise Exception("Symlink points outside jail")
 				realfile = tmp[len(chroot):]
 				jailpath = create_parent_path(chroot, realfile,be_verbose, copy_permissions, allow_suid, copy_ownership)
 		existpath = jailpath
-		#print 'new value for existpath is',existpath
+		#print('new value for existpath is' + existpath)
 		i+=1
 	return existpath
 
@@ -492,20 +514,22 @@ def copy_dir_with_permissions_and_owner(srcdir,dstdir,be_verbose=0):
 	#create directory dstdir
 	try:
 		if (be_verbose):
-			print 'Creating directory'+dstdir
+			print('Creating directory'+dstdir)
 		os.mkdir(dstdir)
 		copy_time_and_permissions(srcdir, dstdir, be_verbose, allow_suid=0, copy_ownership=1)
-	except (IOError, OSError), (errno,strerror):
+	except (IOError, OSError) as e:
+		_, strerror = e.args
 		sys.stderr.write('ERROR: copying directory and permissions '+srcdir+' to '+dstdir+': '+strerror+'\n')
 		return 0
 	for root, dirs, files in os.walk(srcdir):
 		for name in files:
 			if (be_verbose):
-				print 'Copying '+root+'/'+name+' to '+dstdir+'/'+name
+				print('Copying '+root+'/'+name+' to '+dstdir+'/'+name)
 			try:
 				shutil.copyfile(root+'/'+name,dstdir+'/'+name)
 				copy_time_and_permissions(root+'/'+name, dstdir+'/'+name, be_verbose, allow_suid=0, copy_ownership=1)
-			except (IOError,OSError), (errno,strerror):
+			except (IOError,OSError) as e:
+				_, strerror = e.args
 				sys.stderr.write('ERROR: copying file and permissions '+root+'/'+name+' to '+dstdir+'/'+name+': '+strerror+'\n')
 				return 0
 		for name in dirs:
@@ -517,13 +541,14 @@ def move_dir_with_permissions_and_owner(srcdir,dstdir,be_verbose=0):
 	if (retval == 1):
 		# remove the source directory
 		if (be_verbose==1):
-			print 'Removing original home directory '+srcdir
+			print('Removing original home directory '+srcdir)
 		try:
 			shutil.rmtree(srcdir)
-		except (OSError,IOError), (errno,strerror):
+		except (OSError, IOError) as e:
+			_, strerror = e.args
 			sys.stderr.write('ERROR: failed to remove '+srcdir+': '+strerror+'\n')
 	else:
-		print 'Not everything was copied to '+dstdir+', keeping the old directory '+srcdir
+		print('Not everything was copied to '+dstdir+', keeping the old directory '+srcdir)
 
 def copy_with_permissions(src, dst, be_verbose=0, try_hardlink=1, allow_suid=0, retain_owner=0):
 	"""copies/links the file and the permissions (and possibly ownership and setuid/setgid bits"""
@@ -533,13 +558,14 @@ def copy_with_permissions(src, dst, be_verbose=0, try_hardlink=1, allow_suid=0,
 			os.link(src,dst)
 			do_normal_copy = 0
 		except:
-			print 'Linking '+src+' to '+dst+' failed, will revert to copying'
+			print('Linking '+src+' to '+dst+' failed, will revert to copying')
 			pass
 	if (do_normal_copy == 1):
 		try:
 			shutil.copyfile(src,dst)
 			copy_time_and_permissions(src, dst, be_verbose, allow_suid=allow_suid, copy_ownership=retain_owner)
-		except (IOError, OSError), (errno,strerror):
+		except (IOError, OSError) as e:
+			_, strerror = e.args
 			sys.stderr.write('ERROR: copying file and permissions '+src+' to '+dst+': '+strerror+'\n')
 
 def copy_device(chroot, path, be_verbose=1, retain_owner=0):
@@ -547,40 +573,40 @@ def copy_device(chroot, path, be_verbose=1, retain_owner=0):
 	create_parent_path(chroot,os.path.dirname(path), be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=0)	
 	chrootpath = resolve_realpath(chroot+path,chroot)
 	if (os.path.exists(chrootpath)):
-		print 'Device '+chrootpath+' does exist already'
+		print('Device '+chrootpath+' does exist already')
 		return
 	sb = os.stat(path)
 	try:
 		if (sys.platform[:5] == 'linux'):
-			major = sb.st_rdev / 256 #major = st_rdev divided by 256 (8bit reserved for the minor number)
+			major = sb.st_rdev // 256 #major = st_rdev divided by 256 (8bit reserved for the minor number)
 			minor = sb.st_rdev % 256 #minor = remainder of st_rdev divided by 256
 		elif (sys.platform == 'sunos5'):
 			if (sys.maxint == 2147483647):
-				major = sb.st_rdev / 262144 #major = st_rdev divided by 256 (18 bits reserved for the minor number)
+				major = sb.st_rdev // 262144 #major = st_rdev divided by 256 (18 bits reserved for the minor number)
 				minor = sb.st_rdev % 262144 #minor = remainder of st_rdev divided by 256
 			else:
 				#64 bit solaris has 32 bit minor/32bit major
-				major = sb.st_rdev / 2147483647
+				major = sb.st_rdev // 2147483647
 				minor =  sb.st_rdev % 2147483647
 		else:
-			major = sb.st_rdev / 256 #major = st_rdev divided by 256
+			major = sb.st_rdev // 256 #major = st_rdev divided by 256
 			minor = sb.st_rdev % 256 #minor = remainder of st_rdev divided by 256
 		if (stat.S_ISCHR(sb.st_mode)): 
 			mode = 'c'
 		elif (stat.S_ISBLK(sb.st_mode)): 
 			mode = 'b'
 		else:
-			print 'WARNING, '+path+' is not a character or block device'
+			print('WARNING, '+path+' is not a character or block device')
 			return 1
 		if (be_verbose==1):
-			print 'Creating device '+chroot+path
+			print('Creating device '+chroot+path)
 		ret = os.spawnlp(os.P_WAIT, 'mknod','mknod', chrootpath, str(mode), str(major), str(minor))
 		copy_time_and_permissions(path, chrootpath, allow_suid=0, copy_ownership=retain_owner)
 	except:
-		print 'Failed to create device '+chrootpath+', this is a know problem with python 2.1'
-		print 'use "ls -l '+path+'" to find out the mode, major and minor for the device'
-		print 'use "mknod '+chrootpath+' mode major minor" to create the device'
-		print 'use chmod and chown to set the permissions as found by ls -l'
+		print('Failed to create device '+chrootpath+', this is a know problem with python 2.1')
+		print('use "ls -l '+path+'" to find out the mode, major and minor for the device')
+		print('use "mknod '+chrootpath+' mode major minor" to create the device')
+		print('use chmod and chown to set the permissions as found by ls -l')
 
 def copy_dir_recursive(chroot,dir,force_overwrite=0, be_verbose=0, check_libs=1, try_hardlink=1, allow_suid=0, retain_owner=0, handledfiles=[]):
 	"""copies a directory and the permissions recursively, possibly with ownership and setuid/setgid bits"""
@@ -594,7 +620,7 @@ def copy_dir_recursive(chroot,dir,force_overwrite=0, be_verbose=0, check_libs=1,
 				handledfiles = copy_dir_recursive(chroot,tmp,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 			else:
 				files2 += os.path.join(dir, entry),
-		except OSError, e:
+		except OSError as e:
 			sys.stderr.write('ERROR: failed to investigate source file '+tmp+': '+e.strerror+'\n')
 	handledfiles = copy_binaries_and_libs(chroot,files2,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 	return handledfiles 
@@ -625,30 +651,27 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 
 		try:
 			sb = cachedlstat(file)
-		except OSError, e:
+		except OSError as e:
 			if (e.errno == 2):
 				if (try_glob_matching == 1):
 					ret = glob.glob(file)
 					if (len(ret)>0):
 						handledfiles = copy_binaries_and_libs(chroot, ret, force_overwrite, be_verbose, check_libs, try_hardlink=try_hardlink, retain_owner=retain_owner, try_glob_matching=0, handledfiles=handledfiles)
 					elif (be_verbose):
-						print 'Source file(s) '+file+' do not exist'
+						print('Source file(s) '+file+' do not exist')
 				elif (be_verbose):
-					print 'Source file '+file+' does not exist'
+					print('Source file '+file+' does not exist')
 			else:
 				sys.stderr.write('ERROR: failed to investigate source file '+file+': '+e.strerror+'\n')
 			continue
 		# source file exists, resolve the chroot realfile
 		create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=allow_suid, copy_ownership=retain_owner)
 		chrootrfile = resolve_realpath(os.path.normpath(chroot+'/'+file),chroot)
-		#if (rfile in handledfiles):
-		#	create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=retain_owner)
-		# 	continue
 		
 		try:
 			chrootsb = cachedlstat(chrootrfile)
 			chrootfile_exists = 1
-		except OSError, e:
+		except OSError as e:
 			if (e.errno == 2):
 				chrootfile_exists = 0
 			else:
@@ -656,21 +679,21 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 		
 		if ((force_overwrite == 0) and chrootfile_exists and not stat.S_ISDIR(chrootsb.st_mode)):
 			if (be_verbose):
-				print ''+chrootrfile+' already exists, will not touch it'
+				print(''+chrootrfile+' already exists, will not touch it')
 		else:
 			if (chrootfile_exists):
 				if (force_overwrite):
 					if (stat.S_ISREG(chrootsb.st_mode)):
 						if (be_verbose):
-							print 'Destination file '+chrootrfile+' exists, will delete to force update'
+							print('Destination file '+chrootrfile+' exists, will delete to force update')
 						try:
 							os.unlink(chrootrfile)
-						except OSError, e:
-							sys.stderr.write('ERROR: failed to delete '+chroot+rfile+': '+e.strerror+'\ncannot force update '+chroot+rfile+'\n')
+						except OSError as e:
+							sys.stderr.write('ERROR: failed to delete '+chrootrfile+': '+e.strerror+'\ncannot force update '+chrootrfile+'\n')
 							# BUG: perhaps we can fix the permissions so we can really delete the file?
 							# but what permissions cause this error?
 					elif (stat.S_ISDIR(chrootsb.st_mode)):
-						print 'Destination dir '+chrootrfile+' exists'
+						print('Destination dir '+chrootrfile+' exists')
 				else:
 					if (stat.S_ISDIR(chrootsb.st_mode)):
 						pass
@@ -678,16 +701,24 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 						# skip to the next item of the loop
 					else:
 						if (be_verbose):
-							print 'Destination file '+chrootrfile+' exists'
+							print('Destination file '+chrootrfile+' exists')
 						continue
 			create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=allow_suid, copy_ownership=retain_owner)
 			if (stat.S_ISLNK(sb.st_mode)):
 				realfile = os.readlink(file)
+				if (chrootfile_exists):
+					if (be_verbose):
+						print('Destination symlink '+chrootrfile+' exists, will delete to force update')
+					try:
+						os.unlink(chrootrfile)
+					except OSError as e:
+						sys.stderr.write('ERROR: failed to delete '+chrootrfile+': '+e.strerror+'\ncannot force update '+chrootrfile+'\n')
 				try:
-					print 'Creating symlink '+chrootrfile+' to '+realfile
+					print('Creating symlink '+chrootrfile+' to '+realfile)
 					os.symlink(realfile, chrootrfile)
 				except OSError:
-					# if the file exists already
+					# if the file exists already, check if it is correct
+					
 					pass
 				handledfiles.append(file)
 				if (realfile[0] != '/'):
@@ -697,9 +728,9 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 				handledfiles = copy_dir_recursive(chroot,file,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 			elif (stat.S_ISREG(sb.st_mode)):
 				if (try_hardlink):
-					print 'Trying to link '+file+' to '+chrootrfile
+					print('Trying to link '+file+' to '+chrootrfile)
 				else:
-					print 'Copying '+file+' to '+chrootrfile
+					print('Copying '+file+' to '+chrootrfile)
 				copy_with_permissions(file,chrootrfile,be_verbose, try_hardlink, allow_suid, retain_owner)
 				handledfiles.append(file)
 			elif (stat.S_ISCHR(sb.st_mode) or stat.S_ISBLK(sb.st_mode)):
@@ -709,7 +740,7 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 #	in python 2.1 the return value is a tuple, not an object, st_mode is field 0
 #	mode = stat.S_IMODE(sbuf.st_mode)
 			mode = stat.S_IMODE(sb[stat.ST_MODE])
-			if (check_libs and (string.find(file, 'lib') != -1 or string.find(file,'.so') != -1 or (mode & (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)))):
+			if (check_libs and (file.find('lib') != -1 or file.find('.so') != -1 or (mode & (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)))):
 				libs = lddlist_libraries(file)
 				handledfiles = copy_binaries_and_libs(chroot, libs, force_overwrite, be_verbose, 0, try_hardlink, handledfiles=handledfiles)
 	return handledfiles
@@ -719,13 +750,13 @@ def config_get_option_as_list(cfgparser, sectionname, optionname):
 	retval = []
 	if (cfgparser.has_option(sectionname,optionname)):
 		inputstr = cfgparser.get(sectionname,optionname)
-		for tmp in string.split(inputstr, ','):
-			retval += [string.strip(tmp)]
+		for tmp in inputstr.split(','):
+			retval += [tmp.strip()]
 	return retval
 
 def clean_exit(exitno,message,usagefunc,type='ERROR'):
-	print ''
-	print type+': '+message
+	print('')
+	print(type+': '+message)
 	usagefunc()
 	sys.exit(exitno)
 
@@ -733,16 +764,20 @@ def test_numitem_exist(item,num,filename):
 	try:
 		fd = open(filename,'r')
 	except:
-		#print ''+filename+' does not exist'
+		#print(''+filename+' does not exist')
 		return 0
 	line = fd.readline()
+	if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+		line = line.decode('utf-8', 'replace')
 	while (len(line)>0):
-		pwstruct = string.split(line,':')
-		#print 'len pwstruct='+str(len(pwstruct))+' while looking for '+item
+		pwstruct = line.split(':')
+		#print('len pwstruct='+str(len(pwstruct))+' while looking for '+item)
 		if (len(pwstruct) > num and pwstruct[num] == item):
 			fd.close()
 			return 1
 		line = fd.readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 	return 0
 
 def test_user_exist(user, passwdfile):
@@ -767,12 +802,14 @@ def init_passwd_and_group(chroot,users,groups,be_verbose=0):
 	# the chroot passwds file exists, check if any of the users exist already
 			fd2 = open(chroot+'/etc/passwd','r+')
 			line = fd2.readline()
+			if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+				line = line.decode('utf-8', 'replace')
 			while (len(line)>0):
-				pwstruct = string.split(line,':')
+				pwstruct = line.split(':')
 				if (len(pwstruct) >=3):
 					if ((pwstruct[0] in users) or (pwstruct[2] in users)):
 						if (be_verbose):
-							print 'user '+pwstruct[0]+' exists in '+chroot+'/etc/passwd'
+							print('user '+pwstruct[0]+' exists in '+chroot+'/etc/passwd')
 						try:
 							users.remove(pwstruct[0])
 						except ValueError:
@@ -782,20 +819,26 @@ def init_passwd_and_group(chroot,users,groups,be_verbose=0):
 						except ValueError:
 							pass
 				line = fd2.readline()
+				if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+					line = line.decode('utf-8', 'replace')
 			fd2.seek(0,2)
 		if (len(users) > 0):
 			fd = open('/etc/passwd','r')
 			line = fd.readline()
+			if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+				line = line.decode('utf-8', 'replace')
 			while (len(line)>0):
-				pwstruct = string.split(line,':')
+				pwstruct = line.split(':')
 				if (len(pwstruct) >=3):
 					if ((pwstruct[0] in users) or (pwstruct[2] in users)):
 						fd2.write(line)
 						if (be_verbose):
-							print 'writing user '+pwstruct[0]+' to '+chroot+'/etc/passwd'
+							print('writing user '+pwstruct[0]+' to '+chroot+'/etc/passwd')
 						if (not pwstruct[3] in groups):
 							groups += [pwstruct[3]]
 				line = fd.readline()
+				if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+					line = line.decode('utf-8', 'replace')
 			fd.close()
 		fd2.close()
 	# do the same sequence for the group files
@@ -804,12 +847,14 @@ def init_passwd_and_group(chroot,users,groups,be_verbose=0):
 	else:
 		fd2 = open(chroot+'/etc/group','r+')
 		line = fd2.readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 		while (len(line)>0):
-			groupstruct = string.split(line,':')
+			groupstruct = line.split(':')
 			if (len(groupstruct) >=2):
 				if ((groupstruct[0] in groups) or (groupstruct[2] in groups)):
 					if (be_verbose):
-						print 'group '+groupstruct[0]+' exists in '+chroot+'/etc/group'
+						print('group '+groupstruct[0]+' exists in '+chroot+'/etc/group')
 					try:
 						groups.remove(groupstruct[0])
 					except ValueError:
@@ -819,24 +864,30 @@ def init_passwd_and_group(chroot,users,groups,be_verbose=0):
 					except ValueError:
 						pass
 			line = fd2.readline()
+			if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+				line = line.decode('utf-8', 'replace')
 		fd2.seek(0,2)
 	if (len(groups) > 0):
 		fd = open('/etc/group','r')
 		line = fd.readline()
+		if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+			line = line.decode('utf-8', 'replace')
 		while (len(line)>0):
-			groupstruct = string.split(line,':')
+			groupstruct = line.split(':')
 			if (len(groupstruct) >=2):
 				if ((groupstruct[0] in groups) or (groupstruct[2] in groups)):
 					fd2.write(line)
 					if (be_verbose):
-						print 'writing group '+groupstruct[0]+' to '+chroot+'/etc/group'
+						print('writing group '+groupstruct[0]+' to '+chroot+'/etc/group')
 			line = fd.readline()
+			if sys.version_info > (3, 0) and isinstance(line, bytes):                                               #Python 3
+				line = line.decode('utf-8', 'replace')
 		fd.close()
 	fd2.close()
 
 def find_file_in_path(filename):
 	search_path = os.getenv('PATH')
-	paths = string.split(search_path, ':')
+	paths = search_path.split(':')
 	for path in paths:
 		joined = os.path.join(path, filename)
 		if os.path.exists(joined):
