@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Olivier Sessink
+Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013, 2015, 2016, 2021 Olivier Sessink
 All rights reserved.
 
 This file is available under two licences, at your own choice
@@ -123,14 +123,16 @@ static char *expand_executable_w_path(const char *executable, char **allowed_pat
 		while (*path) {
 			char *newpath;
 			int tlen = strlen(*path);
-			newpath = malloc((elen+tlen+2)*sizeof(char));
-			memset(newpath, 0, (elen+tlen+2)*sizeof(char));
-			newpath = strncpy(newpath, *path, tlen);
+			newpath = malloc((elen+tlen+3)*sizeof(char));
+			memset(newpath, 0, (elen+tlen+3)*sizeof(char));
+			newpath = strncpy(newpath, *path, tlen+1); /* the +1 is not needed, but gcc will complain that 
+						the string is not NULL terminated (but it is because of the memset on the previoud line) */
 			if (*(*path+tlen-1) != '/') {
 				newpath = strcat(newpath, "/");
 				DEBUG_LOG("newpath=%s",newpath);
 			}
-			newpath = strncat(newpath, executable, elen);
+			newpath = strncat(newpath, executable, elen+1);  /* the +1 is not needed, but gcc will complain that 
+						the string is not NULL terminated (but it is because of the memset previously) */
 			if (file_exists(newpath)) {
 				DEBUG_MSG("file %s exists\n",newpath);
 				return newpath;
@@ -145,7 +147,28 @@ static char *expand_executable_w_path(const char *executable, char **allowed_pat
 /* returns a NULL terminated array of strings */
 char **expand_newargv(char *string) {
 	wordexp_t p;
-	wordexp(string, &p, 0);
+	int ret;
+	ret = wordexp(string, &p, WRDE_NOCMD);
+	if (ret != 0) {
+		switch(ret) {
+		case WRDE_BADCHAR:
+			syslog(LOG_ERR,"illegal word expansion of %s\n",string);
+		break;
+		case WRDE_BADVAL:
+			syslog(LOG_ERR,"undefined shell variable in %s\n",string);
+		break;
+		case WRDE_CMDSUB:
+			syslog(LOG_ERR,"command substitution not allowed\n");
+		break;
+		case WRDE_NOSPACE:
+			syslog(LOG_ERR,"out of memory\n");
+		break;
+		case WRDE_SYNTAX:
+			syslog(LOG_ERR,"shell syntax error in %s\n",string);
+		break;
+		}
+		exit(6);
+	}
 	return p.we_wordv;
 }
 
@@ -185,8 +208,8 @@ int main (int argc, char **argv) {
 		if (user)
 			syslog(LOG_ERR, "cannot find user info for USER %s: %s", user, strerror(errno));
 		else
-			syslog(LOG_ERR, "cannot find user info for uid %d: %s", getuid(), strerror(errno));
-		DEBUG_MSG(PROGRAMNAME" cannot find user name for uid %d: %s", getuid(), strerror(errno));
+			syslog(LOG_ERR, "cannot find user info for uid %u: %s", getuid(), strerror(errno));
+		DEBUG_MSG(PROGRAMNAME" cannot find user name for uid %u: %s", getuid(), strerror(errno));
 		exit(2);
 	}
 	if (user && pw->pw_uid != getuid()) {
@@ -195,8 +218,8 @@ int main (int argc, char **argv) {
 	}
 	gr = getgrgid(getgid());
 	if (!gr || !gr->gr_name || gr->gr_name[0]=='\0') {
-		syslog(LOG_ERR, "cannot find group name for gid %d: %s", getuid(), strerror(errno));
-		DEBUG_MSG(PROGRAMNAME" cannot find group name for uid %d: %s", getuid(), strerror(errno));
+		syslog(LOG_ERR, "cannot find group name for gid %u: %s", getuid(), strerror(errno));
+		DEBUG_MSG(PROGRAMNAME" cannot find group name for uid %u: %s", getuid(), strerror(errno));
 		exit(3);
 	}
 
@@ -204,8 +227,8 @@ int main (int argc, char **argv) {
 	before that there could be an argument like --login that we simply ignore */
 	if (argc < 3 || strcmp(argv[argc - 2],"-c")!=0) {
 		char *requeststring = implode_array(argv,argc," ");
-		DEBUG_MSG("WARNING: user %s (%d) tried to get an interactive shell session (%s), which is never allowed by jk_lsh\n", pw->pw_name, getuid(),requeststring);
-		syslog(LOG_ERR, "WARNING: user %s (%d) tried to get an interactive shell session (%s), which is never allowed by jk_lsh", pw->pw_name, getuid(),requeststring);
+		DEBUG_MSG("WARNING: user %s (%u) tried to get an interactive shell session (%s), which is never allowed by jk_lsh\n", pw->pw_name, getuid(),requeststring);
+		syslog(LOG_ERR, "WARNING: user %s (%u) tried to get an interactive shell session (%s), which is never allowed by jk_lsh", pw->pw_name, getuid(),requeststring);
 		free(requeststring);
 		exit(7);
 	}
@@ -235,10 +258,10 @@ int main (int argc, char **argv) {
 	DEBUG_MSG("using section %s\n",section);
 
 	DEBUG_MSG("setting umask\n");
-	umaskval = iniparser_get_octalint_at_position(parser, section, "umask", section_pos);
+	umaskval = iniparser_get_octalint_at_position(parser, section, "umask", section_pos, 0000);
 	if (umaskval != -1) {
-		mode_t oldumask;
-		oldumask = umask(umaskval);
+		/*mode_t oldumask;
+		oldumask = */umask(umaskval);
 		/*syslog(LOG_DEBUG, "changing umask from 0%o to 0%o", oldumask, umaskval);*/
 	}
 	if (iniparser_get_string_at_position(parser, section, "environment", section_pos, buffer, 1024) > 0) {
@@ -257,7 +280,7 @@ int main (int argc, char **argv) {
 	}
 
 	DEBUG_MSG("exploding string '%s'\n",argv[argc-1]);
-	if (iniparser_get_int_at_position(parser, section, "allow_word_expansion", section_pos)) {
+	if (iniparser_get_int_at_position(parser, section, "allow_word_expansion", section_pos, 0)) {
 		newargv = expand_newargv(argv[argc-1]);
 	} else {
 		newargv = explode_string(argv[argc-1], ' ');
@@ -279,8 +302,8 @@ int main (int argc, char **argv) {
 	if (!executable_is_allowed(parser, section, newargv[0],section_pos)) {
 		char *logstring;
 		logstring = implode_array(newargv,-1," ");
-		DEBUG_MSG("WARNING: user %s (%d) tried to run '%s'\n", pw->pw_name, getuid(),newargv[0]);
-		syslog(LOG_ERR, "WARNING: user %s (%d) tried to run '%s', which is not allowed according to "CONFIGFILE, pw->pw_name, getuid(),logstring);
+		DEBUG_MSG("WARNING: user %s (%u) tried to run '%s'\n", pw->pw_name, getuid(),newargv[0]);
+		syslog(LOG_ERR, "WARNING: user %s (%u) tried to run '%s', which is not allowed according to "CONFIGFILE, pw->pw_name, getuid(),logstring);
 		free(logstring);
 		exit(4);
 	}
@@ -288,13 +311,13 @@ int main (int argc, char **argv) {
 	iniparser_close(parser);
 
 	logstring = implode_array(newargv,-1," ");
-	DEBUG_MSG("executing command '%s' for user %s (%d)\n", logstring,pw->pw_name, getuid());
-	syslog(LOG_INFO, "executing command '%s' for user %s (%d)", logstring,pw->pw_name, getuid());
+	DEBUG_MSG("executing command '%s' for user %s (%u)\n", logstring,pw->pw_name, getuid());
+	syslog(LOG_INFO, "executing command '%s' for user %s (%u)", logstring,pw->pw_name, getuid());
 	free(logstring);
 	retval = execve(newargv[0],newargv,environ);
 	DEBUG_MSG("errno=%d, error=%s\n",errno,strerror(errno));
 	DEBUG_MSG("execve() command '%s' returned %d\n", newargv[0], retval);
-	syslog(LOG_ERR, "WARNING: running %s failed for user %s (%d): %s", newargv[0],pw->pw_name, getuid(), strerror(retval));
+	syslog(LOG_ERR, "WARNING: running %s failed for user %s (%u): %s", newargv[0],pw->pw_name, getuid(), strerror(retval));
 	syslog(LOG_ERR, "WARNING: check the permissions and libraries for %s", newargv[0]);
 	return retval;
 }
