@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008 Olivier Sessink
+Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2016 Olivier Sessink
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -118,7 +118,7 @@ static Tsocketlink *new_socketlink(int outsocket, char *inpath, int normrate, in
 	sl->insocket = socket(PF_UNIX, SOCK_DGRAM, 0);
 /*	DEBUG_MSG("new_socketlink, insocketserver %s at %d\n", sl->inpath, sl->insocket);*/
 
-	strncpy(serv_addr.sun_path, sl->inpath, sizeof(serv_addr.sun_path));
+	strncpy(serv_addr.sun_path, sl->inpath, sizeof(serv_addr.sun_path)-1);
 	serv_addr.sun_family = PF_UNIX;
 	unlink(sl->inpath);
 	ret = bind(sl->insocket, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
@@ -369,17 +369,15 @@ int main(int argc, char**argv) {
 				prevpos = iniparser_get_position(ip);
 				secpos = prevpos - strlen(tmp)-4;
 				DEBUG_MSG("secpos=%ld, prevpos=%ld\n",secpos,prevpos);
-				base = iniparser_get_int_at_position(ip, tmp, "base", secpos);
-				peak = iniparser_get_int_at_position(ip, tmp, "peak", secpos);
-				interval = iniparser_get_float_at_position(ip, tmp, "interval", secpos);
+				base = iniparser_get_int_at_position(ip, tmp, "base", secpos, 511);
+				peak = iniparser_get_int_at_position(ip, tmp, "peak", secpos, 2048);
+				interval = iniparser_get_float_at_position(ip, tmp, "interval", secpos, 5.0);
 				iniparser_set_position(ip, secpos);
-				if (10 > base || base >  1000000) base = 511;
+				if (10 > base || base >  1000000) {
+					base = 511;
+				}
 				if (100 > peak || peak > 10000000 || peak < base) {
-					/* for backwards compatibility we check 'peek' */
-					peak = iniparser_get_int_at_position(ip, tmp, "peek", secpos);
-					if (100 > peak || peak > 10000000 || peak < base) {
-						peak = 2048;
-					}
+					peak = 2048;
 				}
 				if (0.01 > interval || interval > 60.0) interval = 5.0;
 				sl[numsockets] = new_socketlink(outsocket, tmp, base, peak, (int)(interval*1000000.0), nodetach);
@@ -417,8 +415,13 @@ int main(int argc, char**argv) {
 	}
 
 	if (numsockets == 0) {
-		printf("version "VERSION", no sockets specified in configfile "CONFIGFILE" or on commandline, nothing to do, exiting...\n");
-		syslog(LOG_ERR,"version "VERSION", no sockets specified in configfile "CONFIGFILE" or on commandline, nothing to do, exiting...");
+		int uid = getuid();
+		if (uid!=0) {
+			printf("version "VERSION", you are not root. \n");
+			syslog(LOG_ERR,"version "VERSION", ran by user other than root with UID %u",uid);
+		}
+		printf("version "VERSION", no valid sockets specified in configfile "CONFIGFILE" or on commandline, nothing to do, exiting...\n");
+		syslog(LOG_ERR,"version "VERSION", no sockets valid specified in configfile "CONFIGFILE" or on commandline, nothing to do, exiting...");
 		exit(1);
 	}
 
