@@ -8,7 +8,7 @@
  * group in this shell
  *
 
-Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009 Olivier Sessink
+Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2015, 2016 Olivier Sessink
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -171,11 +171,11 @@ int main(int argc, char **argv) {
 	}
 
 	if (!pw) {
-		syslog(LOG_ERR, "abort, failed to get user information for user ID %d: %s, check /etc/passwd", getuid(), strerror(errno));
+		syslog(LOG_ERR, "abort, failed to get user information for user ID %u: %s, check /etc/passwd", getuid(), strerror(errno));
 		exit(13);
 	}
 	if (!pw->pw_name || strlen(pw->pw_name)==0) {
-		syslog(LOG_ERR, "abort, got an empty username for user ID %d: %s, check /etc/passwd", getuid(), strerror(errno));
+		syslog(LOG_ERR, "abort, got an empty username for user ID %u: %s, check /etc/passwd", getuid(), strerror(errno));
 		exit(13);
 	}
 	if (user && strcmp(user,pw->pw_name)!=0) {
@@ -183,12 +183,12 @@ int main(int argc, char **argv) {
 		exit(13);
 	}
 	if (pw->pw_uid != getuid()) {
-		syslog(LOG_ERR, "abort, started by user ID %d, got user info %s with user ID %d,", getuid(), pw->pw_name, pw->pw_uid);
+		syslog(LOG_ERR, "abort, started by user ID %u, got user info %s with user ID %u,", getuid(), pw->pw_name, pw->pw_uid);
 		exit(13);
 	}
 	gr = getgrgid(getgid());
 	if (!gr) {
-		syslog(LOG_ERR, "abort, failed to get group information for group ID %d: %s, check /etc/group", getgid(), strerror(errno));
+		syslog(LOG_ERR, "abort, failed to get group information for group ID %u: %s, check /etc/group", getgid(), strerror(errno));
 		exit(13);
 	}
 
@@ -281,14 +281,14 @@ int main(int argc, char **argv) {
 				allowed_jails = explode_string(buffer, ',');
 			}
 
-			skip_injail_passwd_check = iniparser_get_int_at_position(parser, section, "skip_injail_passwd_check", pos);
+			skip_injail_passwd_check = iniparser_get_int_at_position(parser, section, "skip_injail_passwd_check", pos, 0);
 
 			free(section);
 		} else {
 			DEBUG_MSG("no relevant section found in configfile\n");
 		}
 		if (allowed_jails == NULL) {
-			syslog(LOG_ERR,"abort, no relevant section for user %s (%d) or group %s (%d) or DEFAULT found in "CONFIGFILE, pw->pw_name,getuid(),gr->gr_name,getgid());
+			syslog(LOG_ERR,"abort, no relevant section for user %s (%u) or group %s (%u) or DEFAULT found in "CONFIGFILE, pw->pw_name,getuid(),gr->gr_name,getgid());
 			exit(1);
 		}
 		iniparser_close(parser);
@@ -330,7 +330,7 @@ int main(int argc, char **argv) {
 			DEBUG_MSG("allowed=%d after testing '%s' with '%s'\n",allowed,jail,allowed_jails[i]);
 		}
 		if (allowed!=1) {
-			syslog(LOG_ERR,"abort, user %s (%d) is not allowed in jail %s",pw->pw_name, getuid(),jail);
+			syslog(LOG_ERR,"abort, user %s (%u) is not allowed in jail %s",pw->pw_name, getuid(),jail);
 			exit(21);
 		}
 	}
@@ -352,7 +352,7 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	syslog(LOG_INFO, "entering jail %s for user %s (%d) in order to execute %s", jail, pw->pw_name, getuid(), executable);
+	syslog(LOG_INFO, "entering jail %s for user %s (%u) in order to execute %s", jail, pw->pw_name, getuid(), executable);
 
 	/* do the chroot() call */
 	if (chroot(jail)) {
@@ -396,11 +396,11 @@ int main(int argc, char **argv) {
 		/* drop all privileges, we first have to setgid(),
 			then we call setuid() */
 		if (setgid(getgid())) {
-			syslog(LOG_ERR, "abort, failed to set effective group ID %d: %s", getgid(), strerror(errno));
+			syslog(LOG_ERR, "abort, failed to set effective group ID %u: %s", getgid(), strerror(errno));
 			exit(34);
 		}
 		if (setuid(getuid())) {
-			syslog(LOG_ERR, "abort, failed to set effective user ID %d: %s", getuid(), strerror(errno));
+			syslog(LOG_ERR, "abort, failed to set effective user ID %u: %s", getuid(), strerror(errno));
 			exit(36);
 		}
 	}
@@ -416,25 +416,25 @@ int main(int argc, char **argv) {
 			pw = getpwuid(getuid());
 		}
 		if (!pw) {
-			syslog(LOG_ERR, "abort, failed to get user information in the jail for user ID %d: %s, check %s/etc/passwd",getuid(),strerror(errno),jail);
+			syslog(LOG_ERR, "abort, failed to get user information in the jail for user ID %u: %s, check %s/etc/passwd",getuid(),strerror(errno),jail);
 			exit(35);
 		}
 		DEBUG_MSG("got %s as pw_dir\n",pw->pw_dir);
 		if (pw->pw_uid != getuid()) {
-			syslog(LOG_ERR, "abort, got user information in the jail for user ID %d instead of user ID %d, check %s/etc/passwd",pw->pw_uid,getuid(),jail);
+			syslog(LOG_ERR, "abort, got user information in the jail for user ID %u instead of user ID %u, check %s/etc/passwd",pw->pw_uid,getuid(),jail);
 			exit(35);
 		}
 		gr = getgrgid(getgid());
 		if (!gr) {
-			syslog(LOG_ERR, "abort, failed to get group information in the jail for group ID %d: %s, check %s/etc/group",getgid(),strerror(errno),jail);
+			syslog(LOG_ERR, "abort, failed to get group information in the jail for group ID %u: %s, check %s/etc/group",getgid(),strerror(errno),jail);
 			exit(35);
 		}
 		if (strcmp(pw->pw_name, oldpw_name)!=0) {
-			syslog(LOG_ERR, "abort, username %s differs from jail username %s for user ID %d, check /etc/passwd and %s/etc/passwd", oldpw_name, pw->pw_name, getuid(), jail);
+			syslog(LOG_ERR, "abort, username %s differs from jail username %s for user ID %u, check /etc/passwd and %s/etc/passwd", oldpw_name, pw->pw_name, getuid(), jail);
 			exit(37);
 		}
 		if (strcmp(gr->gr_name, oldgr_name)!=0) {
-			syslog(LOG_ERR, "abort, groupname %s differs from jail groupname %s for group ID %d, check /etc/passwd and %s/etc/passwd", oldgr_name, gr->gr_name, getgid(), jail);
+			syslog(LOG_ERR, "abort, groupname %s differs from jail groupname %s for group ID %u, check /etc/passwd and %s/etc/passwd", oldgr_name, gr->gr_name, getgid(), jail);
 			exit(37);
 		}
 		free(oldpw_name);
@@ -444,7 +444,7 @@ int main(int argc, char **argv) {
 	/* now execute the jailed shell */
 	execv(executable, newargv);
 	/* normally we wouldn't come to this bit of code */
-	syslog(LOG_ERR, "ERROR: failed to execute %s for user %s (%d), check the permissions and libraries of %s%s",executable,pw->pw_name,getuid(),jail,executable);
+	syslog(LOG_ERR, "ERROR: failed to execute %s for user %s (%u), check the permissions and libraries of %s%s",executable,pw->pw_name,getuid(),jail,executable);
 
 	free(jail);
 	exit(111);
